summaryrefslogtreecommitdiffstats
path: root/pw
diff options
context:
space:
mode:
authorDavid Nugent <davidn@FreeBSD.org>2000-07-13 23:52:49 +0000
committerDavid Nugent <davidn@FreeBSD.org>2000-07-13 23:52:49 +0000
commita7bda97c1f46bcacc464f9f1c5fa8c697f9574a8 (patch)
tree686466a631b769b6ad0b50627dc9f30acd8171d6 /pw
parent483c94ce774634296cb0d083008ba2358ab6b6a1 (diff)
downloadpw-darwin-a7bda97c1f46bcacc464f9f1c5fa8c697f9574a8.tar.gz
pw-darwin-a7bda97c1f46bcacc464f9f1c5fa8c697f9574a8.tar.zst
pw-darwin-a7bda97c1f46bcacc464f9f1c5fa8c697f9574a8.zip
Support propogation of file flags when building a homedir from the
skeleton directory. sprintf() -> snprintf() cleanup. PR: bin/8756 Submitted by: "Nickolay N. Dudorov" <nnd@itfs.nsk.su>
Diffstat (limited to 'pw')
-rw-r--r--pw/cpdir.c23
1 files changed, 17 insertions, 6 deletions
diff --git a/pw/cpdir.c b/pw/cpdir.c
index 6d59401..d8fa42b 100644
--- a/pw/cpdir.c
+++ b/pw/cpdir.c
@@ -72,18 +72,23 @@ copymkdir(char const * dir, char const * skel, mode_t mode, uid_t uid, gid_t gid
while ((e = readdir(d)) != NULL) {
char *p = e->d_name;
- sprintf(src, "%s/%s", skel, p);
- if (stat(src, &st) == 0) {
+ if (snprintf(src, sizeof(src), "%s/%s", skel, p) >= (int)sizeof(src))
+ warn("warning: pathname too long '%s/%s' (skel not copied)", skel, p);
+ else if (stat(src, &st) == 0) {
if (strncmp(p, "dot.", 4) == 0) /* Conversion */
p += 3;
- sprintf(dst, "%s/%s", dir, p);
- if (S_ISDIR(st.st_mode)) { /* Recurse for this */
+ if (snprintf(dst, sizeof(dst), "%s/%s", dir, p) >= (int)sizeof(dst))
+ warn("warning: path too long '%s/%s' (skel file skipped)", dir, p);
+ else {
+ if (S_ISDIR(st.st_mode)) { /* Recurse for this */
if (strcmp(e->d_name, ".") != 0 && strcmp(e->d_name, "..") != 0)
copymkdir(dst, src, (st.st_mode & 0777), uid, gid);
+ chflags(dst, st.st_flags); /* propogate flags */
/*
* Note: don't propogate 'special' attributes
*/
- } else if (S_ISREG(st.st_mode) && (outfd = open(dst, O_RDWR | O_CREAT | O_EXCL, st.st_mode)) != -1) {
+ } else if (S_ISREG(st.st_mode) && (outfd = open(dst, O_RDWR | O_CREAT | O_EXCL, st.st_mode)) != -1) {
if ((infd = open(src, O_RDONLY)) == -1) {
close(outfd);
remove(dst);
@@ -98,9 +103,15 @@ copymkdir(char const * dir, char const * skel, mode_t mode, uid_t uid, gid_t gid
while ((b = read(infd, copybuf, 4096)) > 0)
write(outfd, copybuf, b);
close(infd);
+ /*
+ * Propogate special filesystem flags
+ */
+ fchown(outfd, uid, gid);
+ fchflags(outfd, st.st_flags);
close(outfd);
chown(dst, uid, gid);
}
+ }
}
}
}
@@ -72,18 +72,23 @@ copymkdir(char const * dir, char const * skel, mode_t mode, uid_t uid, gid_t gid
/*
- * Note: don't propogate 'special' attributes
+ * Note: don't propogate special attributes
+ * but do propogate file flags
*/